@@ -1,9 +1,9 @@
-import requests,json,time
+import requests,json,time,datetime
 
 #Created and maintained by Kristan Smout
 #Github URL = https://github.com/KristanSmout/RobloxPyOfficial
 
-Version = '0.0.96'
+Version = '0.0.99'
 
 
 APIURL = "https://api.roblox.com/"
@@ -18,6 +18,7 @@ UserAPI = APIURL + "users/"
 UserAPIV1 = 'https://users.roblox.com/v1/users/'
 GroupAPI = APIURL + "groups/"
 GroupAPIV1 = "https://groups.roblox.com/v1/groups/"
+PrivateMessageAPIV1 = "https://privatemessages.roblox.com/v1/"
 
 
 #Internal
@@ -101,8 +102,107 @@ def IsBanned(UserID):
    response = requests.get(UserAPIV1 + str(UserID))
    return response.json()['isBanned']
 
+def UserCreationDate(UserID,WantedData):
+    WantedData = WantedData.lower()
+    CreationDate = requests.get(UserAPIV1 + str(UserID)).json()['created']
+    CreationDate = CreationDate.split('T')
+    CreationDate = CreationDate[0].split('-')
+    
+    if(WantedData == ('year'.lower())):
+        return str(CreationDate[0])
+    elif(WantedData == 'month'.lower()):
+        return str(CreationDate[1])
+    elif(WantedData == 'day'.lower()):
+        return str(CreationDate[2])
+        
+
+def AccountAgeDays(UserID):
+    CreationDate = requests.get(UserAPIV1 + str(UserID)).json()['created']
+    CreationDate = CreationDate.split('T')
+    CreationDate = CreationDate[0].split('-')
+    CurrentDate = datetime.date.today()
+    CreationDate = datetime.date(int(CreationDate[0]),int(CreationDate[1]),int(CreationDate[2]))
+    Days = ((datetime.date.today()) - (CreationDate))
+    Days = str(Days).split(' ')
+    return Days[0]
+
+def GetFollowingCount(UserID):
+    response = requests.get(FriendsURL + 'users/' + str(UserID) + '/followings/count')
+    try:
+        return response.json()['count']
+    except:
+        return 'UserID Invalid'
+
+def GetFollowersCount(UserID):
+    response = requests.get(FriendsURL + 'users/' + str(UserID) + '/followers/count')
+    try:
+        return response.json()['count']
+    except:
+        return 'UserID Invalid'
+    
+def GetFollowers(UserID,Ammount):
+    response = requests.get(FriendsURL + '/users/' + str(UserID) + '/followers?sortOrder=Asc&limit=100')
+    CurrentAmmount = 0
+    NameList = []
+    IDList = []
+    Cursor = 'None'
+    while Cursor !=  'null':
+        if(Cursor is None):
+            Cursor = 'null'
+        else:
+            try:
+                Cursor = response.json()['nextPageCursor']
+            except:
+                Cursor = 'null'
+        for Follower in response.json()['data']:
+            if(CurrentAmmount < Ammount):
+                NameList.append(Follower['name'])
+                IDList.append(Follower['id'])
+                CurrentAmmount = CurrentAmmount + 1
+            else:
+                return NameList,IDList
+        if(Cursor is None):
+            Cursor = 'null'
+        else:
+            try:
+                response = requests.get(FriendsURL + '/users/' + str(UserID) + '/followers?sortOrder=Asc&limit=100&cursor=' + str(Cursor))
+            except:
+                Cursor = 'null'
+    return NameList,IDList
+
+def GetFollowing(UserID,Ammount):
+    response = requests.get(FriendsURL + '/users/' + str(UserID) + '/followings?sortOrder=Asc&limit=100')
+    CurrentAmmount = 0
+    NameList = []
+    IDList = []
+    Cursor = 'None'
+    while Cursor !=  'null':
+        if(Cursor is None):
+            Cursor = 'null'
+        else:
+            try:
+                Cursor = response.json()['nextPageCursor']
+            except:
+                Cursor = 'null'
+        for Follower in response.json()['data']:
+            if(CurrentAmmount < Ammount):
+                NameList.append(Follower['name'])
+                IDList.append(Follower['id'])
+                CurrentAmmount = CurrentAmmount + 1
+            else:
+                return NameList,IDList
+        if(Cursor is None):
+            Cursor = 'null'
+        else:
+            try:
+                response = requests.get(FriendsURL + '/users/' + str(UserID) + '/followers?sortOrder=Asc&limit=100&cursor=' + str(Cursor))
+            except:
+                Cursor = 'null'
+    return NameList,IDList
+
+
 #endregion
-IsBanned(1)
+
 #region RAP
 
 def GetUserRAP(UserID):
@@ -148,12 +248,12 @@ def GetUserTerribleDemandLimiteds(UserID):
 
 def IsGroupOwned(GroupID):
     response = requests.get(GroupAPIV1 + str(GroupID))
-    
     if(str(response.json()['owner']) == 'None'):
         return False
     else:
         return True
 
+
 def GetGroupName(GroupID):
     response = requests.get(GroupAPIV1 + str(GroupID))
     return response.json()['name']
@@ -347,7 +447,11 @@ def IsFollowing(Cookie,UserID):
 def FollowUser(Cookie,UserID):
     session = SetCookie(Cookie)
     Post = session.post('https://friends.roblox.com/v1/users/' + str(UserID) + '/follow',data={'targetUserID': UserID})
-    return Post.json()['success']
+    data = Post.json()
+    try:
+        return data['success']
+    except:
+        return data['errors']
 
 def UnfollowUser(Cookie,UserID):
     session = SetCookie(Cookie)
@@ -386,15 +490,17 @@ def TotalFriends(Cookie):
     response = session.get(FriendsURL + 'my/friends/count')
     return response.json()['count']
 
-def SendMessage(Cookie,UserID,MessageSubject,Body):
+def SendMessage(Cookie,UserID,MessageSubject,Body): #Credit to Soul for documentation URL
+    LocalID = GetUserID(Cookie)
     session = SetCookie(Cookie)
-    Post = session.post('https://www.roblox.com/messages/send',data={
-                       'subject': MessageSubject,
-                       'body': Body,
-                       'recipientid': str(UserID),
-                       'cacheBuster': str(int(time.time()))
+    Post = session.post(PrivateMessageAPIV1 + 'messages/send/' ,data={
+                        'userId' : LocalID,
+                        'subject': MessageSubject,
+                        'body': Body,
+                        'recipientid': UserID,
                })
-    return Post
+    return Post.json()['shortMessage']
+    
 
 def GetBlockedUsers(Cookie):
     session = SetCookie(Cookie)
